#include "DepotBase.h"

DepotBase::DepotBase() : Depot() {

}

DepotBase::DepotBase(OBJHANDLE cBody) : Depot() {

	_cBody = cBody;
}

DepotBase::~DepotBase(void)
{
	_expositions.clear();
}

DepotBasePtr DepotBase::Create(OBJHANDLE cBody) {

#ifdef USE_SMART_PTRS
	return make_shared<DepotBase>(cBody);
#else
	return new DepotBase(cBody);
#endif
}

#ifdef USE_SMART_PTRS
DepotBasePtr DepotBase::Null = DepotBasePtr();
#else
DepotBasePtr DepotBase::Null = NULL;
#endif

void DepotBase::AddStorage(StoragePtr storage, ExpositionBase exposition) {

	Depot::AddStorage(storage);
	//_expositions.insert(make_pair(storage.GetProduct().GetId(), exposition));
	_expositions[storage->GetProduct()->GetId()] = exposition;
}

int DepotBase::Unload(const int& storageIndex, const int& amount, char* customName) {

	// check that storage index is valid
	StoragePtr storage = GetStorage(storageIndex);
	if (!storage) {
		return STORAGE_NOT_FOUND;
	}

	// check that storage is not empty
	int toExpose = min(storage->GetStored(), amount);
	toExpose = max(toExpose, 0);
	if (toExpose == 0) {
		return NOTHING_TO_EXPOSE;
	}

	char fileName[100];

	// check that vessel is installed
	sprintf(fileName, "Config\\Vessels\\%s.cfg", storage->GetProduct()->GetVessel().data());
	if (_access(fileName, 0) == -1) {
		return VESSEL_NOT_INSTALLED;
	}

	int customNameLength = strlen(customName);
	bool exactName = amount == 1 && customNameLength > 0;

	// check that a vessel with custom name does not exist
	if (exactName > 0 && oapiGetVesselByName(customName) != NULL) {
		return VESSEL_NAME_EXISTS;
	}

	char vesselName[100];

	// find already existing in the scenario vessels with the default product name,
	// their indexes and positions
	vector<UINT> reservedIndexes;
	vector<VECTOR3> reservedPositions;
	DWORD vesselCount = oapiGetVesselCount();

	string defaultProductName = storage->GetDefaultVesselName();
	if (defaultProductName.length() == 0) {
		defaultProductName = storage->GetProduct()->GetId();
	}
	UINT defaultProductNameLength = defaultProductName.length();
	UINT index;
	//double speed;
	for (DWORD vesselIndex = 0; vesselIndex < vesselCount; vesselIndex++) {

		OBJHANDLE hVessel = oapiGetVesselByIndex(vesselIndex);
		oapiGetObjectName(hVessel, vesselName, 100);

		if (!strnicmp(vesselName, defaultProductName.data(), defaultProductNameLength) 
				|| (customNameLength > 0 && !strnicmp(vesselName, customName, customNameLength))) {

			if (!exactName) {
				if (customNameLength > 0) {
					if (strlen(vesselName) == customNameLength + 3) {
						sscanf(vesselName + customNameLength + 1, "%d", &index);
						reservedIndexes.push_back(index);
					}
				} else if (strlen(vesselName) == defaultProductNameLength + 3) {
					sscanf(vesselName + defaultProductNameLength + 1, "%d", &index);
					reservedIndexes.push_back(index);
				}
			}

			//oapiGetAirspeed(hVessel, &speed);
			//if (IsNear(speed, 0.0, 0.1)) {
				VECTOR3 pos;
				oapiGetEquPos(hVessel, &(pos.x), &(pos.y), &(pos.z));
				reservedPositions.push_back(pos);
			//}
		}
	}

	ExpositionBase exposition = _expositions[storage->GetProduct()->GetId()];
	double cBodyRadius = oapiGetSize(_cBody);
	double radSlotLength = storage->GetProduct()->GetSlotLength() / cBodyRadius;
	double radSlotWidth = (storage->GetProduct()->GetSlotWidth() / cBodyRadius);// / cos(exposition.lat * RAD);
	VECTOR3 rotateAxis = {0, 0, -1};

	for (int i = 0; i < toExpose; i++) {

		if (reservedIndexes.size() >= 100) {
			return ALL_INDEXES_USED;
		}

		VESSELSTATUS2 vs;
		memset (&vs, 0, sizeof(vs));
		vs.version = 2;
		vs.status = 1; // landed
		vs.rbody = _cBody;

		vs.surf_hdg = exposition.hdg * RAD;

		// find name
		if (!exactName) {
			for (UINT index = 0; index < 100; index++) {
				bool exists = false;
				for (vector<UINT>::iterator iter = reservedIndexes.begin(); iter != reservedIndexes.end(); ++iter) {
					if (index == *iter) {
						exists = true;
						break;
					}
				}
				if (!exists) {
					sprintf(vesselName, "%s-%02d", customNameLength > 0 ? customName : defaultProductName.data(), index);
					reservedIndexes.push_back(index);
					break;
				}
			}
		} else {
			sprintf(vesselName, "%s", customName);
		}

		// find place
		bool found = false;
		for (UINT x = 0; x < 10; x++) {
			for (UINT y = 0; y < 10; y++) {
				VECTOR3 pos;
				pos.x = x * radSlotWidth;
				pos.y = -(y * radSlotLength);
				pos.z = 0.0;
				RotateVector(pos, rotateAxis, exposition.hdg * RAD);
				pos.x /= cos(exposition.lat * RAD);
				pos.x += exposition.lng * RAD;
				pos.y += exposition.lat * RAD;
				pos.z = cBodyRadius;

				bool occupied = false;
				for (vector<VECTOR3>::iterator iter = reservedPositions.begin(); iter != reservedPositions.end(); ++iter) {
					double distance = oapiOrthodome(iter->x, iter->y, pos.x, pos.y);
					if (distance * cBodyRadius < 0.5) {
						occupied = true;
						break;
					}
				}
				if (!occupied) {
					vs.surf_lng = pos.x;
					vs.surf_lat = pos.y;
					reservedPositions.push_back(pos);
					found = true;
				}
				if (found) break;
			}
			if (found) break;
		}

		if (found) {
			// create the vessel
			OBJHANDLE hVessel = oapiCreateVesselEx (vesselName, storage->GetProduct()->GetVessel().data(), &vs);
			if (hVessel != NULL) {
				storage->Remove(1);

				// load default state
				sprintf(fileName, "Config\\DefStates\\%s.txt", storage->GetProduct()->GetId().data());
				if (_access(fileName, 0) != -1) {
					FILEHANDLE file = oapiOpenFile(fileName, FILE_IN);
					if (file != NULL) {
						try {
							VESSEL2* vessel2 = (VESSEL2*) oapiGetVesselInterface(hVessel);
							//VESSEL2* vessel2 = static_cast<VESSEL2*>(vessel);
							vessel2->clbkLoadStateEx(file, &vs);
							oapiCloseFile(file, FILE_IN);

							if (XRVesselCtrl::IsXRVesselCtrl(vessel2)) {
								XRVesselCtrl* xrVessel = static_cast<XRVesselCtrl*>(vessel2);
								if (xrVessel->GetCtrlAPIVersion() >= THIS_XRVESSELCTRL_API_VERSION) {
									oapiSetFocusObject(hVessel);
									xrVessel->SetDoorState(XRDoorID::XRD_APU, XRDoorState::XRDS_Open);
									vessel2->clbkPostStep(0, 0, 0);
									xrVessel->SetDoorState(XRDoorID::XRD_APU, XRDoorState::XRDS_Closed);
								}
							}
						} catch (...) {
							oapiWriteLog("Plants: Exception on load state");
						}
					}
				}
				if (oapiGetPause()) {
					oapiSetPause(false);
				}
			}
		} else {
			return NO_PLACE;
		}
	}
	return ALL_EXPOSED;
}

/*
double d = sqrt (x * x + z * z);
double sindR_d = sin (d / R) / d;
double cosdR = cos (d / R);
double sinlat = sin (base_lat);
double coslat = cos (base_lat);
double lat = asin (sinlat * cosdR - x * coslat * sindR_d);
double lng = base_lng + atan2 (z * sindR_d * coslat, cosdR - sinlat * sin (lat));
*/

bool DepotBase::Load(const int& storageIndex) {

	StoragePtr storage = GetStorage(storageIndex);
	if (!storage || storage->IsFull()) {
		return false;
	}

	ExpositionBase exposition = _expositions[storage->GetProduct()->GetId()];
	double cBodyRadius = oapiGetSize(_cBody);
	double radAreaLength = storage->GetProduct()->GetSlotLength() * 10.0 / cBodyRadius;
	double radAreaWidth = storage->GetProduct()->GetSlotWidth() * 10.0 / cBodyRadius;
	double meterLengthRad = 1.0 / cBodyRadius;
	double meterWidthRad = 1.0 * cos(exposition.lat * RAD) / cBodyRadius;

	VECTOR3 areaStartPoint;
	areaStartPoint.x = exposition.lng * RAD;
	areaStartPoint.y = exposition.lat * RAD;
	areaStartPoint.z = 0;

	DWORD vesselCount = oapiGetVesselCount();
	VESSELSTATUS2 vs;
	vs.version = 2;
	VECTOR3 rotateAxis = {0, 0, -1};
	VECTOR3 pos;
	string vesselClass;

	for (DWORD vesselIndex = 0; vesselIndex < vesselCount; vesselIndex++) {
		OBJHANDLE hVessel = oapiGetVesselByIndex(vesselIndex);
		if (oapiGetEmptyMass(hVessel) < 0.001) {
			continue;
		}
		VESSEL* vessel = oapiGetVesselInterface(hVessel);
		string vesselClass = vessel->GetClassName();
		if (storage->GetProduct()->GetVessel().compare(vesselClass) == 0) {
			vessel->GetStatusEx(&vs);
			if (vs.status == 1 && vs.rbody == _cBody) {
			
				pos.x = vs.surf_lng;
				pos.y = vs.surf_lat;
				pos.z = 0;

				pos -= areaStartPoint;
				pos.x *= cos(exposition.lat * RAD);
				RotateVector(pos, rotateAxis, -exposition.hdg * RAD);

				if (pos.x >= (0 - meterWidthRad) && pos.x <= (radAreaWidth + meterWidthRad) 
						&& pos.y <= (0 + meterLengthRad) && pos.y >= (-radAreaLength - meterLengthRad)) {
					if (oapiDeleteVessel(vessel->GetHandle())) {
						vessel->SetEmptyMass(0.0001);
						storage->Add(1);
						return true;
					}
				}
			}
		}
	}
	return false;
}

void DepotBase::SetCBody(OBJHANDLE cBody) {

	_cBody = cBody;
}

void DepotBase::SetBaseName(string baseName) {

	_baseName = baseName;
}

string DepotBase::GetBaseName() {

	return _baseName;
}
