#include "Plant.h"

Plant::Plant() {

	_rate = 0.0;
	_rateInSec = 0.0;
	_progress = 0.0;
	_resourceCheckDt = 0.0;
}

Plant::~Plant(void) {
	
}

void Plant::Produce(double simdt) {

	if (!InProduction()) {
		_resourceCheckDt += simdt;
		if (_resourceCheckDt > RESOURCE_CHECK_INTERVAL) {
			_resourceCheckDt -= RESOURCE_CHECK_INTERVAL;
			if (!StartNewProduction()) {
				return;
			}
		} else {
			return;
		}
	}

	_progress += _rateInSec * simdt;
	if (_progress > 1.0) {
		StoragePtr storage = GetStorage(_product->GetId());
		if (storage && storage->GetStored() < storage->GetCapacity()) {
			storage->Add(1);
			if (StartNewProduction()) {
				_progress -= 1.0;
			} else {
				_progress = 0.0;
			}
		} else {
			_progress = 1.0;
		}
	}
}

bool Plant::InProduction() {

	return _progress != 0.0;
}

double Plant::GetRemainingTime() {

	return (1.0 - _progress) / _rateInSec;
}

bool Plant::StartNewProduction() {

	map<string, int>& resources = _product->GetResources();
	bool enough = true;

	for (map<string, int>::iterator iter = resources.begin(); iter != resources.end(); iter++) {
		StoragePtr storage = GetStorage(iter->first);
		if (!storage || storage->GetStored() < iter->second) {
			enough = false;
			break;
		}
	}

	if (enough) {
		for (map<string, int>::iterator iter = resources.begin(); iter != resources.end(); iter++) {
			GetStorage(iter->first)->Remove(iter->second);
		}
		return true;
	} else {
		return false;
	}
}

void Plant::SetRate(double rate) {

	_rate = rate;
	_rateInSec = _rate / SECONDS_IN_DAY;
}

void Plant::SetProgress(double progress) {

	_progress = progress;
}

float Plant::GetProgress() {

	return _progress;
}
