#pragma once

#include "Common.h"
#include "Utils.h"
#include "ModuleAPI.h"
#include "PlantBase.h"
#include "Product.h"

void UpdateDialogPlantInfo();

namespace oapi {

const double PRODUCTION_STEP = 1.0;

class Plants : public Module {
public:

	Plants(HINSTANCE hDLL);
	~Plants();
	void clbkSimulationStart (RenderMode mode);
	void clbkSimulationEnd ();
	void clbkPostStep (double simt, double simdt, double mjd);

	void ConsumeScenarioLine(string line);
	void SaveState(FILEHANDLE scn);

	bool SetSelectedCBodyIndex(FacilityType facilityType, int cBodyIndex);
	OBJHANDLE GetSelectedCBody() {return _selectedCBody;};
	vector<DepotBasePtr>& GetSelectedCBodyFacilities(FacilityType facilityType);

	bool SetSelectedFacilityIndex(FacilityType facilityType, int index);
	PlantBasePtr GetSelectedPlant();
	DepotBasePtr GetSelectedDepot();
	DepotBasePtr GetSelectedFacility(FacilityType fType);

	int GetCBodiesWithFacilitiesCount(FacilityType facilityType);
	vector<OBJHANDLE> GetCBodies(FacilityType facilityType);

	int UnloadSelectedFacility(FacilityType locationType, const int& storageIndex, const int& amount,
		char* customName);
	bool LoadSelectedFacility(FacilityType locationType, const int& storageIndex);

private:
	
	vector<ProductPtr> _products;

	// map of plants to cBody
	map<OBJHANDLE, vector<DepotBasePtr>> _plants;
	map<OBJHANDLE, vector<DepotBasePtr>> _depots;

	vector<string> _scenarioLines;

	OBJHANDLE _selectedCBody;
	PlantBasePtr _selectedPlant;
	DepotBasePtr _selectedDepot;
	double _productionDt;

	OBJHANDLE _focusObject;

	void LoadProducts();
	ProductPtr GetProduct(string id);

	void LoadCBodyFacilities(FacilityType facilityType, OBJHANDLE cBody);
	void SaveFacilitiesState(FacilityType facilityType, FILEHANDLE scn);

	map<OBJHANDLE, vector<DepotBasePtr>>& GetFacilities(FacilityType facilityType);
	DepotBasePtr GetFacility(FacilityType facilityType, const string& cBodyName, const string& depotName);
	vector<DepotBasePtr>& GetCBodyFacilities(FacilityType facilityType, OBJHANDLE cBody);
};

};
