#define STRICT
#define ORBITER_MODULE

#include "windows.h"
#include "Commctrl.h"
#include "Plants.h"
#include "PlantBase.h"
#include "resource.h"
#include <stdio.h>

using namespace oapi;

// ==============================================================
// Global variables
// ==============================================================

HINSTANCE g_hInst;  // module instance handle
DWORD g_dwCmd;      // custom function identifier
Plants* g_plants;	// module instance
HWND g_hDlg;

typedef struct {
	//HFONT hFont[3];
	//HPEN hPen[5];
	HBRUSH hBrush[1];
} GDIParams;
GDIParams g_gdiParams;

FacilityType g_facilityType = PLANT_BASE;

static const COLORREF DARK_GREEN = RGB(0, 128, 0);
static const COLORREF RED = RGB(255, 0, 0);
static const COLORREF ORANGE = RGB(200, 200, 0);

COLORREF g_remainingTimeColor = DARK_GREEN;

// ==============================================================
// Local prototypes
// ==============================================================

void OpenDlgClbk(void *context);
BOOL CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);

void StartLeakMonitor();
void LogLeaks();

// ==============================================================
// API interface
// ==============================================================

// ==============================================================
// This function is called when Orbiter starts or when the module
// is activated.

DLLCLBK void InitModule(HINSTANCE hDLL) {

	StartLeakMonitor();

	g_hInst = hDLL; // remember the instance handle

	g_gdiParams.hBrush[0] = CreateSolidBrush(GetSysColor(COLOR_MENU));

	g_dwCmd = oapiRegisterCustomCmd("Plants", "Manage plants all over the world", OpenDlgClbk, NULL);

	g_plants = new Plants(hDLL);
	oapiRegisterModule(g_plants);
}

// ==============================================================
// This function is called when Orbiter shuts down or when the
// module is deactivated

DLLCLBK void ExitModule(HINSTANCE hDLL) {

	oapiUnregisterCustomCmd(g_dwCmd);

	for (int i = 0; i < 1; i++) DeleteObject(g_gdiParams.hBrush[i]);

	LogLeaks();
}

// ==============================================================
// Write some parameters to the scenario file

DLLCLBK void opcSaveState(FILEHANDLE scn) {

	g_plants->SaveState(scn);
}

// ==============================================================
// Read custom parameters from scenario

DLLCLBK void opcLoadState(FILEHANDLE scn) {

	char *line;
	while (oapiReadScenario_nextline(scn, line)) {
		g_plants->ConsumeScenarioLine(line);
	}
}

// ==============================================================
// Open the dialog window

void OpenDlgClbk(void *context) {

	g_hDlg = oapiOpenDialog(g_hInst, IDD_MYDIALOG, DlgProc);
}

// ==============================================================
// Close the dialog

void CloseDlg(HWND hDlg) {

	oapiCloseDialog(hDlg);
	g_hDlg = NULL;
}

// ==============================================================
// Show the actual info about the selected plant.

void UpdateDialogPlantInfo(HWND hDlg, bool newSelection) {

	if (hDlg != NULL) {

		HWND hList = GetDlgItem(hDlg, IDC_STORAGELIST);

		if (newSelection) {
			ListView_DeleteAllItems(hList);
			SetWindowText(GetDlgItem(hDlg, IDC_MSG), "");
			SetWindowText(GetDlgItem(hDlg, IDC_CUSTOMNAME), "");
			SetWindowText(GetDlgItem(hDlg, IDC_UNLOADAMOUNT), "");
		}

		DepotBasePtr selectedFacility = g_plants->GetSelectedFacility(g_facilityType);
		if (selectedFacility) {
			char buf[256];

			if (selectedFacility->IsPlant()) {
				PlantBasePtr plant = PlantBase::Cast(selectedFacility);
				if (newSelection) {
					SetWindowText(GetDlgItem (hDlg, IDC_PRODUCTNAME), 
						plant->GetProduct()->GetName().data());
				}
				SendDlgItemMessage(hDlg, IDC_PROGRESS, PBM_SETPOS, 
					plant->GetProgress() * 100, 0);
				if (plant->InProduction()) {
					g_remainingTimeColor = DARK_GREEN;
					int remainingTime = (int)ceil(plant->GetRemainingTime());
					//sprintf(buf, "Finish at %0.5f. Remaining time: %dd %dh %dm", oapiTime2MJD(oapiGetSimTime() + remainingTime),
					sprintf(buf, "Remaining time: %dd %dh %dm %ds",
						remainingTime / SECONDS_IN_DAY, 
						(remainingTime % SECONDS_IN_DAY) / SECONDS_IN_HOUR,
						(remainingTime % SECONDS_IN_HOUR) / 60,
						remainingTime % 60);
					SetWindowText(GetDlgItem(hDlg, IDC_REMAININGTIME), buf);
				} else {
					g_remainingTimeColor = RED;
					SetWindowText(GetDlgItem(hDlg, IDC_REMAININGTIME), "Not enough resources!");
				}
			}

			if (selectedFacility->GetStorageCount() > 0) {

				int storageCount = selectedFacility->GetStorageCount();

				if (ListView_GetItemCount(hList) != storageCount) {
					// create item for each storage
					LVITEM item;
					item.mask = LVIF_TEXT;
					item.iSubItem = 0;
					//item.pszText = LPSTR_TEXTCALLBACK;
					for (int i = 0; i < storageCount; i++) {
						item.iItem = i;
						string text = selectedFacility->GetStorage(i)->GetProduct()->GetName();
						item.pszText = (LPSTR)text.data();
						ListView_InsertItem(hList, &item);
					}
				}

				if (selectedFacility->IsPlant()) {

					map<string, int>& resources = PlantBase::Cast(selectedFacility)->GetProduct()->GetResources();

					// show actual storage state
					for (int i = 0; i < storageCount; i++) {
						StoragePtr storage = selectedFacility->GetStorage(i);
						map<string, int>::iterator pos = resources.find(storage->GetProduct()->GetId());
						if (pos != resources.end()) {
							sprintf(buf, "%d / %d (%d)", storage->GetStored(), storage->GetCapacity(), pos->second);
						} else {
							sprintf(buf, "%d / %d", storage->GetStored(), storage->GetCapacity());
						}
						ListView_SetItemText(hList, i, 1, buf);
					}
				} else {
					// show actual storage state
					for (int i = 0; i < storageCount; i++) {
						StoragePtr storage = selectedFacility->GetStorage(i);
						sprintf(buf, "%d / %d", storage->GetStored(), storage->GetCapacity());
						ListView_SetItemText(hList, i, 1, buf);
					}
				}
			}
		}
	}
}

// ==============================================================
// May be used to update the selected plant info if there is no 
// dialog handle.

void UpdateDialogPlantInfo() {

	UpdateDialogPlantInfo(g_hDlg, false);
}

// ==============================================================
// Fill plants combobox for the selected cBody.

void OnCBodySelect(HWND hDlg) {

	SendDlgItemMessage(hDlg, IDC_COMBOPLANTS, CB_RESETCONTENT, 0, 0);

	vector<DepotBasePtr>& cBodyFacilities = g_plants->GetSelectedCBodyFacilities(g_facilityType);
	DepotBasePtr selectedFacility = g_plants->GetSelectedFacility(g_facilityType);

	int selectIndex = 0;
	if (cBodyFacilities.size() > 0) {
		int index = 0;
		for (vector<DepotBasePtr>::iterator iter = cBodyFacilities.begin(); iter != cBodyFacilities.end(); ++iter) {
			SendDlgItemMessage(hDlg, IDC_COMBOPLANTS, CB_ADDSTRING, 0, (LPARAM)(*iter)->GetName().data());
			if (selectedFacility && selectedFacility->GetName().compare((*iter)->GetName()) == 0) {
				selectIndex = index;
			}
			index++;
		}
	}

	SendDlgItemMessage (hDlg, IDC_COMBOPLANTS, CB_SETCURSEL, selectIndex, 0);
	if (g_plants->SetSelectedFacilityIndex(g_facilityType, selectIndex)) {
		UpdateDialogPlantInfo(hDlg, true);
	}
}

void ShowFacilities(HWND hDlg, FacilityType fType) {

	SetWindowText(GetDlgItem(hDlg, IDC_PRODUCTNAME), "");
	SetWindowText(GetDlgItem(hDlg, IDC_MSG), "");
	SetWindowText(GetDlgItem(hDlg, IDC_CUSTOMNAME), "");
	SetWindowText(GetDlgItem(hDlg, IDC_UNLOADAMOUNT), "");

	ShowWindow(GetDlgItem(hDlg, IDC_REMAININGTIME), SW_HIDE);

	SendDlgItemMessage (hDlg, IDC_COMBOCBODIES, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage (hDlg, IDC_COMBOPLANTS, CB_RESETCONTENT, 0, 0);

	ListView_DeleteAllItems(GetDlgItem(hDlg, IDC_STORAGELIST));

	ShowWindow(GetDlgItem(hDlg, IDC_PRODUCTTEXT), SW_HIDE);
	ShowWindow(GetDlgItem(hDlg, IDC_PROGRESSTEXT), SW_HIDE);
	ShowWindow(GetDlgItem(hDlg, IDC_PROGRESS), SW_HIDE);

	if (g_plants->GetCBodiesWithFacilitiesCount(fType) > 0) {

		if (fType == PLANT_BASE || fType == PLANT_VESSEL) {
			ShowWindow(GetDlgItem(hDlg, IDC_PRODUCTTEXT), SW_SHOW);
			ShowWindow(GetDlgItem(hDlg, IDC_PROGRESSTEXT), SW_SHOW);
			ShowWindow(GetDlgItem(hDlg, IDC_PROGRESS), SW_SHOW);
			ShowWindow(GetDlgItem(hDlg, IDC_REMAININGTIME), SW_SHOW);
		}

		// find current cBody handle
		OBJHANDLE hFocusObject = g_plants->GetSelectedCBody();
		if (hFocusObject == NULL) {
			hFocusObject = oapiGetFocusObject();
			if (oapiIsVessel(hFocusObject)) {
				hFocusObject = oapiGetVesselInterface(hFocusObject)->GetSurfaceRef();
			}
		}

		vector<OBJHANDLE> cBodies = g_plants->GetCBodies(fType);
		int selectedIndex = 0;
		int index = 0;
		for (vector<OBJHANDLE>::iterator iter = cBodies.begin(); iter != cBodies.end(); ++iter) {

			OBJHANDLE cBodyHandle = *iter;

			char name[100];
			oapiGetObjectName(cBodyHandle, name, 100);
			SendDlgItemMessage (hDlg, IDC_COMBOCBODIES, CB_ADDSTRING, 0, (LPARAM) name);

			if (cBodyHandle == hFocusObject) {
				selectedIndex = index;
			}
			index++;
		}

		SendDlgItemMessage (hDlg, IDC_COMBOCBODIES, CB_SETCURSEL, selectedIndex, 0);
		if (g_plants->SetSelectedCBodyIndex(fType, selectedIndex)) {
			OnCBodySelect(hDlg);
		}

	} else {

		g_plants->SetSelectedFacilityIndex(fType, -1);
	}
}

void InitListView(HWND hDlg) {

	HWND hList = GetDlgItem (hDlg, IDC_STORAGELIST);

	ListView_SetExtendedListViewStyle(hList, LVS_EX_FULLROWSELECT | LVS_EX_AUTOSIZECOLUMNS | LVS_EX_GRIDLINES);

	LVCOLUMN column;
	column.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	column.iSubItem = 0;

	column.pszText = "Product";
	column.cx = 180;
	ListView_InsertColumn(hList, 0, &column);

	column.pszText = "Stored";
	column.cx = 100;
	ListView_InsertColumn(hList, 1, &column);
}

void InitTabControl(HWND hDlg) {

	HWND hTabCtrl = GetDlgItem(hDlg, IDC_TABCTRL);

	TCITEM tie;
	tie.mask = TCIF_TEXT | TCIF_IMAGE; 
    tie.iImage = -1; 

    tie.pszText = "Plants"; 
	TabCtrl_InsertItem(hTabCtrl, 0, &tie);
    tie.pszText = "Depots"; 
	TabCtrl_InsertItem(hTabCtrl, 1, &tie);

	TabCtrl_SetCurSel(hTabCtrl, g_facilityType);
}

// ==============================================================
// Windows message handler for the dialog box.

BOOL CALLBACK DlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:

		InitListView(hDlg);
		InitTabControl(hDlg);

		ShowFacilities(hDlg, g_facilityType);

		return TRUE;

	case WM_CTLCOLORSTATIC:

		if ((HWND)lParam == GetDlgItem(hDlg, IDC_REMAININGTIME)) {
			if (g_facilityType == PLANT_BASE || g_facilityType == PLANT_VESSEL) {
				SetTextColor((HDC)wParam, g_remainingTimeColor);
				SetBkMode((HDC)wParam, TRANSPARENT);
				return (BOOL)g_gdiParams.hBrush[0];
			}
		}
		break;

	case WM_DESTROY:
		return TRUE;

	case WM_COMMAND:

		switch (LOWORD (wParam)) {

		case IDCANCEL: // dialog closed by user
			CloseDlg (hDlg);
			return TRUE;

		case IDC_COMBOCBODIES:
			if (HIWORD(wParam) == CBN_SELCHANGE) {
				int index = SendMessage((HWND)lParam, CB_GETCURSEL, 0, 0);
				if (index != CB_ERR && g_plants->SetSelectedCBodyIndex(g_facilityType, index)) {
					OnCBodySelect(hDlg);
				}
			}
			return TRUE;

		case IDC_COMBOPLANTS:
			if (HIWORD(wParam) == CBN_SELCHANGE) {
				int index = SendMessage((HWND)lParam, CB_GETCURSEL, 0, 0);
				if (index != CB_ERR && g_plants->SetSelectedFacilityIndex(g_facilityType, index)) {
					UpdateDialogPlantInfo(hDlg, true);
				}
			}
			return TRUE;

		case IDC_UNLOAD:
			if (HIWORD(wParam) == BN_CLICKED) {

				int selectedIndex = ListView_GetSelectionMark(GetDlgItem(hDlg, IDC_STORAGELIST));
				if (selectedIndex < 0) {
					selectedIndex = 0;
				}
				char sAmount[20];
				GetWindowText(GetDlgItem(hDlg, IDC_UNLOADAMOUNT), sAmount, 20);
				int sAmountLength = strlen(sAmount);
				if (sAmountLength > 0 && sAmountLength < 4) {
					int amount;
					sscanf(sAmount, "%d", &amount);
					// is custom vessel name specified
					char customName[50];
					GetWindowText(GetDlgItem(hDlg, IDC_CUSTOMNAME), customName, 50);
					if (g_plants->UnloadSelectedFacility(g_facilityType, selectedIndex, amount, customName) 
							!= VESSEL_NAME_EXISTS) {
						SetWindowText(GetDlgItem(hDlg, IDC_MSG), "");
					} else {
						SetWindowText(GetDlgItem(hDlg, IDC_MSG), "Vessel with this name already exists");
					}
				}
				return TRUE;
			}
			break;

		case IDC_LOAD:
			if (HIWORD(wParam) == BN_CLICKED) {

				int selectedIndex = ListView_GetSelectionMark(GetDlgItem(hDlg, IDC_STORAGELIST));
				if (selectedIndex >= 0) {
					SetWindowText(GetDlgItem(hDlg, IDC_MSG), "");
					g_plants->LoadSelectedFacility(g_facilityType, selectedIndex);
				} else {
					SetWindowText(GetDlgItem(hDlg, IDC_MSG), "Select a storage");
				}
				return TRUE;
			}
			break;

		case IDC_LOADALL:
			if (HIWORD(wParam) == BN_CLICKED) {

				int selectedIndex = ListView_GetSelectionMark(GetDlgItem(hDlg, IDC_STORAGELIST));
				if (selectedIndex >= 0) {
					SetWindowText(GetDlgItem(hDlg, IDC_MSG), "");
					while (g_plants->LoadSelectedFacility(g_facilityType, selectedIndex)) {
					};
				} else {
					SetWindowText(GetDlgItem(hDlg, IDC_MSG), "Select a storage");
				}
				return TRUE;
			}
			break;
		}
		break;

	case WM_NOTIFY:
		if (((LPNMHDR)lParam)->code == TCN_SELCHANGE) {

			int selection = TabCtrl_GetCurSel(GetDlgItem(hDlg, IDC_TABCTRL));
			g_facilityType = selection == 0 ? PLANT_BASE : DEPOT_BASE;
			ShowFacilities(hDlg, g_facilityType);
		}
		break;
	}
	return oapiDefDialogProc(hDlg, uMsg, wParam, lParam);
}



#ifdef _DEBUG

static _CrtMemState state;

void StartLeakMonitor()
{
    _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_CHECK_ALWAYS_DF);
    _CrtMemCheckpoint(&state);
}

void LogLeaks()
{
    OutputDebugString("--- Start Leak Dump ---\n");
    _CrtMemDumpAllObjectsSince(&state);
}

#else

void StartLeakMonitor()
{
}

void LogLeaks()
{
}

#endif
